from setuptools import setup


def get_long_description():
    with open('README.md') as readme:
        return readme.read()


setup(
    name='pearun',
    version='1.0.0',
    packages=['pearun'],
    author='Milan Vlasák',
    author_email='krakenus02@gmail.com',
    description='A simple utility to run user defined commands',
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url='https://github.com/Krakenus/pearun',
    license='MIT',
    python_requires='>=3.6',
    install_requires=[
        'click',
    ],
    entry_points={
        'console_scripts': [
            'pearun = pearun.__main__:main',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



import sys
import subprocess
import os

import click

from pearun.exceptions import PearunException, UnspecifiedCommandException, PearunfileException
from pearun.pearunfile import Pearunfile

__all__ = ['main']


def _execute_command(command, cwd) -> None:
    """
    Runs specified command
    :param command: command to run
    :param cwd: dir containing Pearunfile for cwd change so relative paths from Pearunfile are valid
    """
    os.chdir(cwd)
    process = subprocess.Popen(command, shell=True)
    process.wait()


def _show_help() -> None:
    with click.Context(main) as context:
        help = context.get_help()
        print(help)


def _list_commands(pearunfile: Pearunfile) -> None:
    if len(pearunfile):
        print('Available commands:')
        for command_name, command in pearunfile.commands.items():
            print('    {:<20}{:<40}'.format(command_name, command))


def _show_help_and_list_commands(pearunfile: Pearunfile) -> None:
    _show_help()
    print('')
    _list_commands(pearunfile)


@click.command(help='Tool for running user defined scripts')
@click.option(
    '-h',
    '--help',
    default=False,
    is_flag=True,
    help='Show this message and exit'
)
@click.option(
    '-f',
    '--file',
    default='Pearunfile',
    show_default=True,
    envvar='PEARUNFILE',
    type=click.Path(),
    help='Specify Pearunfile path'
)
@click.option(
    '-l',
    '--list',
    default=False,
    is_flag=True,
    help='List available commands'
)
@click.argument(
    'command',
    type=click.STRING,
    required=False
)
@click.argument(
    'args',
    nargs=-1,
    type=click.STRING
)
def main(**kwargs):
    if kwargs.get('help', False):
        _show_help()
        sys.exit(0)

    file_path = kwargs.get('file')

    try:
        pearunfile = Pearunfile(file_path)
    except PearunException as e:
        print(e, end='\n\n')
        _show_help()
        sys.exit(1)
    except PearunfileException as e:
        print(e)
        sys.exit(1)

    if kwargs.get('list', False):
        _list_commands(pearunfile)
        sys.exit(0)

    command_name = kwargs.get('command')
    args = kwargs.get('args', [])

    try:
        command = pearunfile.resolve_command(command_name, *args)
    except UnspecifiedCommandException:
        _show_help_and_list_commands(pearunfile)
        sys.exit(1)
    except PearunException as e:
        print(e, end='\n\n')
        _list_commands(pearunfile)
        sys.exit(1)

    cwd = os.path.dirname(file_path)
    _execute_command(command, cwd)


if __name__ == '__main__':
    main()



from collections import OrderedDict
import json
import typing
import os

from pearun.exceptions import PearunfileException, PearunException, UnspecifiedCommandException


class Pearunfile:

    def __init__(self, path: str, lazy: bool = False):
        self.path: str = path
        self._check_file_exists()
        self._commands: typing.Optional[OrderedDict] = None if lazy else self._parse_commands()

    def _check_file_exists(self) -> None:
        if not os.path.exists(self.path):
            raise PearunException('File {} not exists'.format(self.path))

    def _parse_commands(self) -> OrderedDict:
        """
        parses a json Pearunfile to dict and makes it sorted
        :return: OrderedDict of defined commands
        """
        with open(self.path) as file:
            try:
                commands = json.load(file)
            except Exception as e:
                raise PearunfileException('Pearunfile parsing failed: {}'.format(e))
        if not all(isinstance(command, str) for command in commands.values()):
            raise PearunfileException('Pearunfile validation failed: commands are not string values')
        return OrderedDict(sorted(commands.items()))

    @property
    def commands(self) -> OrderedDict:
        """
        :return:
        """
        if not self._commands:
            self._commands = self._parse_commands()
        return self._commands

    def resolve_command(self, command_name: str, *args) -> str:
        """
        :param command_name:
        :param args:
        :return:
        """

        if not command_name:
            raise UnspecifiedCommandException()

        try:
            command = self[command_name]
        except KeyError:
            raise PearunException('Unrecognized command: {}'.format(command_name))
        command = ' '.join([command, *args])
        return command

    def __bool__(self) -> bool:
        return bool(self.commands)

    def __len__(self) -> int:
        return len(self.commands)

    def __getitem__(self, item: str) -> str:
        return self.commands[item]

    def __contains__(self, item: str) -> bool:
        return item in self.commands
